@@ -1,175 +1,7 @@
 import numpy as np
+from .functions import *
+from .evaluation import *
 
-class Layer:
-    def __init__(self):
-        self.inputs = None
-        self.targets = None
-        self.outputs = None
-        self.dX = None
-        self.dY = None
-        self.dW = None
-        self.dB = None
-
-    def derivative(self):
-        pass
-    def forward(self):
-        pass
-    def backward(self):
-        pass
-
-class Loss:
-    def loss(self):
-        pass
-    def derivative(self):
-        pass
-    
-#%%Cost Functions
-class Binary_CrossEntropy(Loss):   
-    def __init__(self):
-        super().__init__()
-        
-    def loss(self, targets, outputs):
-        e = 1e-10
-        return -(np.sum(targets * np.log(outputs + e) + (1-targets) * np.log(1-outputs + e))) / len(targets)
-    
-    def derivative(self, targets, outputs):
-        e = 1e-10
-        return -(targets / outputs - (1 - targets + e) / (1 - outputs + e)) / len(outputs) 
-    
-class Categorical_CrossEntropy(Loss):
-    def __init__(self):
-        super().__init__()
-        
-    def loss(self, targets, outputs):
-        targets_clipped = np.clip(outputs, 1e-7, 1 - 1e-7)
-        if len(targets.shape) == 1:
-            correct_confidences = targets_clipped[
-            range(len(outputs)),
-            targets
-            ]
-        elif len(targets.shape) == 2:
-            correct_confidences = np.sum(
-            targets_clipped*targets,
-            axis=1
-            )
-        negative_log_likelihoods = -np.log(correct_confidences)
-        return np.mean(negative_log_likelihoods)
-    
-    def derivative(self, targets, outputs):
-        if len(targets.shape) == 1:
-            targets = np.eye(len(outputs[0]))[targets]
-        return ((-targets + 1e-5) / (outputs + 1e-5)) / len(outputs) 
-
-#%%Activation Functions
-class Dense(Layer):
-    def __init__(self, n_in, n_out):
-        super().__init__()
-        self.weights = np.random.rand(n_in, n_out)
-        self.biases = np.random.rand(1,n_out)
-        self.w_momentum = np.zeros_like(self.weights)
-        self.b_momentum = np.zeros_like(self.biases)
-        
-    def forward(self):
-        self.outputs = np.dot(self.inputs, self.weights) + self.biases
-        return np.dot(self.inputs, self.weights) + self.biases
-    
-    def backward(self, dX, lr, momentum):
-        self.dY = dX
-        self.dW = np.dot(self.inputs.T, self.dY)
-        self.dB = np.sum(self.dY, axis=0, keepdims=True)
-        self.dX = np.dot(self.dY, self.weights.T)
-        
-        if momentum is not None:
-            weights = momentum * self.w_momentum - self.dW * lr
-            biases = momentum * self.b_momentum - self.dB * lr
-            
-            self.w_momentum = weights
-            self.b_momentum = biases
-            self.weights += weights
-            self.biases += biases
-            
-        elif momentum is None:
-            self.weights -= self.dW * lr
-            self.biases -= self.dB * lr
-        
-        self.dW *= np.zeros(self.dW.shape, dtype = np.float64)
-        self.dB *= np.zeros(self.dB.shape, dtype = np.float64)
-        
-        return self.dX
-    
-class ReLU(Layer):
-    def __init__(self):
-        super().__init__()
-        
-    def derivative(self):
-        self.inputs[self.inputs<=0] = 0
-        self.inputs[self.inputs>0] = 1
-        return self.inputs
-        
-    def forward(self):
-        self.outputs = np.maximum(0,self.inputs)
-        return np.maximum(0,self.inputs)
-    
-    def backward(self, dX, lr, momentum):
-        self.dY = dX
-        self.dX = self.dY * self.derivative()
-        return self.dX
-
-class Leaky_ReLU(Layer):
-    def __init__(self):
-        super().__init__()
-    
-    def derivative(self):
-        dA = np.ones_like(self.inputs)
-        dA[self.inputs < 0] = 0.01
-        return dA
-    
-    def forward(self):
-        self.outputs = np.where(self.inputs > 0, self.inputs, self.inputs * 0.01)
-        return np.where(self.inputs > 0, self.inputs, self.inputs * 0.01)
-    
-    def backward(self, dX, lr, momentum):
-        self.dY = dX
-        self.dX = self.dY * self.derivative()
-        return self.dX
-    
-class Sigmoid(Layer):
-    def __init__(self):
-        super().__init__()
-    
-    def derivative(self):
-        return self.outputs * (1-self.outputs)
-    
-    def forward(self):
-        self.outputs = 1 / (1+np.exp(-self.inputs))
-        return 1 / (1+np.exp(-self.inputs))
-    
-    def backward(self, dX, lr, momentum):
-        self.dY = dX
-        self.dX = self.dY * self.derivative()
-        return self.dX
-
-class Softmax(Layer):
-    def __init__(self):
-        super().__init__()
-        self.dK = None
-        
-    def forward(self):
-        exp_values = np.exp(self.inputs - np.max(self.inputs, axis=1,keepdims=True))
-        probabilities = exp_values / np.sum(exp_values, axis=1,keepdims=True)
-        self.outputs = probabilities
-        return self.outputs
-    
-    def backward(self, dX, lr, momentum):
-        dA = np.empty_like(dX)
-        for index, (single_output, single_dvalues) in enumerate(zip(self.outputs, dX)):
-            single_output = single_output.reshape(-1, 1)
-            jacobian_matrix = np.diagflat(single_output) - np.dot(single_output, single_output.T)
-            dA[index] = np.dot(jacobian_matrix,
-            single_dvalues)
-        return dA
-
-#%%LiveStats
 class LiveStats():
     def __init__(self, type=-1, f1=0, f2=1, row_select="limited"):
         super().__init__()
@@ -282,7 +114,6 @@ class LiveStats():
             figure.canvas.draw()
             figure.canvas.flush_events()
             
-#%%Sequential 
 class Sequential():
     def __init__(self, train_inputs, train_targets, val_inputs = None, val_targets = None):
         self.train_inputs = train_inputs
@@ -344,13 +175,13 @@ class Sequential():
                     self.val_layers[i+1].inputs = self.val_layers[i].forward()
                 
     def predict(self,X):
-            self.train_layers[0].inputs = X
-            for i in range(len(self.train_layers)):
-                if i+1 == len(self.train_layers):
-                    self.prediction = np.around(self.train_layers[i].forward())
-                    return self.train_layers[i].forward()
-                else:
-                    self.train_layers[i+1].inputs = self.train_layers[i].forward()
+        self.train_layers[0].inputs = X
+        for i in range(len(self.train_layers)):
+            if i+1 == len(self.train_layers):
+                self.prediction = self.train_layers[i].forward()
+                return self.train_layers[i].forward()
+            else:
+                self.train_layers[i+1].inputs = self.train_layers[i].forward()
             
     def backward(self, lr, momentum):
         self.dX = self.train_cost_fn.derivative(self.train_targets, self.train_outputs)
@@ -361,19 +192,6 @@ class Sequential():
                 if isinstance(self.val_layers[i], Dense):
                     self.val_layers[i].weights = self.train_layers[i].weights
                     self.val_layers[i].biases = self.train_layers[i].biases
-            
-    def accuracy(self, targets, outputs):
-        accuracy = 1e-10
-        outputs = np.array(outputs)
-        if outputs.shape[1] > 1:
-            for i in range(outputs.shape[0]):
-                outputs[i][np.argmax(outputs[i], axis=0)] = 1.0
-        for i in range(outputs.shape[1]):
-            if i+2 > outputs.shape[1]:
-                accuracy = np.array(np.sum(np.around(outputs[:,i]) == targets[:,i]) / len(targets[:,i]))
-                return accuracy
-            else:
-                accuracy = np.array(np.sum(np.around(outputs[:,i]) == targets[:,i]) / len(targets[:,i])) * np.array(np.sum(np.around(outputs[:,i+1]) == targets[:,i+1]) / len(targets[:,i+1]))
         
     def verbose(self,verbose,epoch,epochs,start):
         import time
@@ -381,30 +199,30 @@ class Sequential():
         if verbose == 2:
             if epoch+1 < epochs:
                 if self.val_inputs is not None and self.val_targets is not None:
-                    print(f"[{epoch+1}/{epochs}] | train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} | val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(self.accuracy(self.val_targets,self.val_outputs),5)}\n")
+                    print(f"[{epoch+1}/{epochs}] | train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} | val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(accuracy(self.val_targets,self.val_outputs),5)}\n")
                 else:
-                    print(f"[{epoch+1}/{epochs}] | train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} | train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)}\n")
+                    print(f"[{epoch+1}/{epochs}] | train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} | train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)}\n")
             elif epoch+1 == epochs:
                 if self.val_inputs is not None and self.val_targets is not None:
                     self.runtime = time.time() - start
-                    line_length1 = len(f"| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} |")
-                    line_length2 = len(f"| val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(self.accuracy(self.val_targets,self.val_outputs),5)}")
-                    print(line_length1 * "-" + f"\n| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} |\n" + line_length1 * "-" + f"\n| val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(self.accuracy(self.val_targets,self.val_outputs),5)}" + (line_length1-line_length2-1) * " " + "|\n" + line_length1 * "-")
+                    line_length1 = len(f"| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} |")
+                    line_length2 = len(f"| val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(accuracy(self.val_targets,self.val_outputs),5)}")
+                    print(line_length1 * "-" + f"\n| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} |\n" + line_length1 * "-" + f"\n| val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(accuracy(self.val_targets,self.val_outputs),5)}" + (line_length1-line_length2-1) * " " + "|\n" + line_length1 * "-")
                     print(f"{round(self.runtime, 5)} seconds")
                 else:
                     self.runtime = time.time() - start
-                    line_length1 = len(f"| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} |")
-                    print(line_length1 * "-" + f"\n| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} |\n" + line_length1 * "-")
+                    line_length1 = len(f"| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} |")
+                    print(line_length1 * "-" + f"\n| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} |\n" + line_length1 * "-")
                     print(f"{round(self.runtime, 5)} seconds")
         elif verbose == 1:
             if epoch+1 == epochs:
                 if self.val_inputs is not None and self.val_targets is not None:
-                    line_length1 = len(f"| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} |")
-                    line_length2 = len(f"| val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(self.accuracy(self.val_targets,self.val_outputs),5)}")
-                    print("\n" + line_length1 * "-" + f"\n| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} |\n" + line_length1 * "-" + f"\n| val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(self.accuracy(self.val_targets,self.val_outputs),5)}" + (line_length1-line_length2-1) * " " + "|\n" + line_length1 * "-")
+                    line_length1 = len(f"| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} |")
+                    line_length2 = len(f"| val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(accuracy(self.val_targets,self.val_outputs),5)}")
+                    print("\n" + line_length1 * "-" + f"\n| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} |\n" + line_length1 * "-" + f"\n| val_loss: {np.around(self.val_cost_fn.loss(self.val_targets, self.val_outputs),5)} · val_accuracy: {np.around(accuracy(self.val_targets,self.val_outputs),5)}" + (line_length1-line_length2-1) * " " + "|\n" + line_length1 * "-")
                 else:
-                    line_length = len(f"| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} |")
-                    print("\n" + line_length * "-" + f"\n| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(self.accuracy(self.train_targets,self.train_outputs),5)} |\n" + line_length * "-")
+                    line_length = len(f"| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} |")
+                    print("\n" + line_length * "-" + f"\n| train_loss: {np.around(self.train_cost_fn.loss(self.train_targets, self.train_outputs),5)} · train_accuracy: {np.around(accuracy(self.train_targets,self.train_outputs),5)} |\n" + line_length * "-")
         elif verbose == 0:
             return 
         else:
@@ -472,11 +290,11 @@ class Sequential():
                 self.forward()
                 if epoch % update == 0 or epoch == 1:
                     self.train_loss_records.append(self.train_cost_fn.loss(self.train_targets, self.train_outputs))
-                    self.train_accuracy_records.append(self.accuracy(self.train_targets, self.train_outputs))
+                    self.train_accuracy_records.append(accuracy(self.train_targets, self.train_outputs))
                 if self.val_inputs is not None and self.val_targets is not None:
                     if epoch % update == 0 or epoch == 1:
                         self.val_loss_records.append(self.val_cost_fn.loss(self.val_targets, self.val_outputs))
-                        self.val_accuracy_records.append(self.accuracy(self.val_targets, self.val_outputs))
+                        self.val_accuracy_records.append(accuracy(self.val_targets, self.val_outputs))
                 self.backward(lr, momentum)
                 
                 if live_stats.type != 0:
@@ -492,11 +310,11 @@ class Sequential():
                 self.forward()
                 if epoch % update == 0 or epoch == 1:
                     self.train_loss_records.append(self.train_cost_fn.loss(self.train_targets, self.train_outputs))
-                    self.train_accuracy_records.append(self.accuracy(self.train_targets, self.train_outputs))
+                    self.train_accuracy_records.append(accuracy(self.train_targets, self.train_outputs))
                 if self.val_inputs is not None and self.val_targets is not None:
                     if epoch % update == 0 or epoch == 1:
                         self.val_loss_records.append(self.val_cost_fn.loss(self.val_targets, self.val_outputs))
-                        self.val_accuracy_records.append(self.accuracy(self.val_targets, self.val_outputs))
+                        self.val_accuracy_records.append(accuracy(self.val_targets, self.val_outputs))
                 self.backward(lr, momentum)
                 
                 if live_stats.type != 0:
