@@ -1,39 +1,87 @@
 class RobustScaler():
+    def __init__(self):
+        self.fit = False
+        self.q1_list = []
+        self.med_list = []
+        self.q3_list = []
+        
     def transform(self, X):
         import numpy as np
-        self.X = None
         self.X = np.copy(X)
-        for i in range(len(self.X[0][:])):
-           q1 = np.quantile(self.X[:,i], 0.25)
-           median = np.quantile(self.X[:,i], 0.5)
-           q3 = np.quantile(self.X[:,i], 0.75)
-           for k in range(len(self.X[:][:])):
-               self.X[k,i] = (self.X[k,i] - median) / (q3 - q1)
-        return self.X
+        if self.fit == False:
+            for i in range(len(self.X[0][:])):
+               q1 = np.quantile(self.X[:,i], 0.25)
+               self.q1_list.append(q1)
+               median = np.quantile(self.X[:,i], 0.5)
+               self.med_list.append(median)
+               q3 = np.quantile(self.X[:,i], 0.75)
+               self.q3_list.append(q3)
+               for k in range(len(self.X[:][:])):
+                   self.X[k,i] = (self.X[k,i] - median) / (q3 - q1)
+            self.fit = True
+            return self.X
+        else:
+            for i in range(len(self.X[0][:])):
+               q1 = self.q1_list[i]
+               median = self.med_list[i]
+               q3 = self.q3_list[i]
+               for k in range(len(self.X[:][:])):
+                   self.X[k,i] = (self.X[k,i] - median) / (q3 - q1)
+            return self.X
 
 class MinMaxScaler():
+    def __init__(self):
+        self.fit = False
+        self.min_list = []
+        self.max_list = []
+        
     def transform(self, X):
         import numpy as np
-        self.X = None
         self.X = np.copy(X)
-        for i in range(len(self.X[0][:])):
-           minimum = np.amin(self.X[:,i])
-           maximum = np.amax(self.X[:,i])
-           for k in range(len(self.X[:][:])):
-               self.X[k,i] = (self.X[k,i] - minimum) / (maximum - minimum)
-        return self.X
+        if self.fit == False:
+            for i in range(len(self.X[0][:])):
+               minimum = np.amin(self.X[:,i])
+               self.min_list.append(minimum)
+               maximum = np.amax(self.X[:,i])
+               self.max_list.append(maximum)
+               for k in range(len(self.X[:][:])):
+                   self.X[k,i] = (self.X[k,i] - minimum) / (maximum - minimum)
+            self.fit = True
+            return self.X
+        else:
+            for i in range(len(self.X[0][:])):
+               minimum = self.min_list[i]
+               maximum =self.max_list[i]
+               for k in range(len(self.X[:][:])):
+                   self.X[k,i] = (self.X[k,i] - minimum) / (maximum - minimum)
+            return self.X
 
 class StandardScaler():
+    def __init__(self):
+        self.fit = False
+        self.std_list = []
+        self.mean_list = []
+        
     def transform(self, X):
         import numpy as np
-        self.X = None
         self.X = np.copy(X)
-        for i in range(len(self.X[0][:])):
-           std = np.std(self.X[:,i])
-           mean = np.mean(self.X[:,i])
-           for k in range(len(self.X[:][:])):
-               self.X[k,i] = (self.X[k,i] - mean) / (std)
-        return self.X
+        if self.fit == False:
+            for i in range(len(self.X[0][:])):
+               std = np.std(self.X[:,i])
+               self.std_list.append(std)
+               mean = np.mean(self.X[:,i])
+               self.mean_list.append(mean)
+               for k in range(len(self.X[:][:])):
+                   self.X[k,i] = (self.X[k,i] - mean) / (std)
+            self.fit = True
+            return self.X
+        else:
+            for i in range(len(self.X[0][:])):
+               std = self.std_list[i]
+               mean = self.mean_list[i]
+               for k in range(len(self.X[:][:])):
+                   self.X[k,i] = (self.X[k,i] - mean) / (std)
+            return self.X
 
 class OneHotEncoder():
     def __init__(self):
@@ -147,7 +195,6 @@ class FeatureEncoder():
             for i in range(len(self.feature_indices)):
                 temp = self.encoders[i].transform(self.X[:,self.feature_indices[i]])
                 self.X = np.delete(self.X, self.feature_indices[i], axis=1)
-                print(temp)
                 for k in range(len(temp[0,:])):
                     self.X = np.insert(self.X, self.feature_indices[i]+k, temp[:,k], axis=1)
             return self.X     
